// ********************************************************************
// VNC1L-1A USB-Hostcontroller, USB-Stick am Mikrokontroller verwenden
// SPI-Kommunikationstest durch E(cho)-Funktion
// AVR-Version / GGC 4.3.2
// Version 1.0 / Stand: 01.11.2009
// AVR-Version / GGC 4.3.2
// Mikrocontroller: ATmega644 (andere mglich)
// Frequenz: 1MHz
// VNC1L-Interface: SPI (per Software)
// ********************************************************************
// FLASH-Speicherbedarf: ca. 450 Byte
// SRAM-Speicherbedarf:  ca.   2 Byte
// ********************************************************************
// Der USB-Stick muss mit FAT oder FAT32 formatiert sein. Es wird nur
// eine primre Partion untersttzt. NTFS Systeme funktionieren nicht.
// ********************************************************************
// Das Programm testet die Kommunikation zwischen dem MC und dem VNC1L
// bzw. VDIP1-Modul per SPI-Schnittstelle. Dazu wird die Echo-Funktion
// benutzt. Ein E + CR wird zum VNC1L gesendet. Dieser sendet das 
// E + CR als Echo unmittelbar zurck.
// Zu Beginn wird die Bereitschaft des VNC1L abgewartet. Dieser sendet
// nach dem Reset einige Statusinformation zur geladenen Firmware und
// ob ein USB-Stick eingesteckt und erkannt wurde. Der Vorgang endet
// mit dem Senden eines > (Prompt) gefolgt von CR. Danach ist der
// VNC1L zur weiteren Kommunikation bereit.
// Hinweis: Es muss ein USB-Stick angesteckt sein!!!
// Wenn alles OK ist, sollte die LED nach ca. 5 Sekunden mit blinken,
// etwa im Sekundentakt beginnen.
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// Includebereich
//************************************************************************
#define F_CPU 1000000UL
#include <avr/io.h>         // IO-Grundfunktionen
#include <string.h>		    // Stringfunktion
#include <stdio.h>		    // Standardfunktion
#include <stdlib.h>         // Standardbibliothek
#include <util/delay.h>     // Pasuenfunktionen

//************************************************************************
// Definebereich ANPASSEN
//************************************************************************

// Portverwendung ATmega644
// LED: PB0
// VNC: SCLK PD3
//      SDI  PD4
//      SDO  PD5
//      CS   PD6
//      RS   PD7

#define LED_D1_EIN  PORTB &= ~(1<<PB0)     /* LED */
#define LED_D1_AUS  PORTB |=  (1<<PB0)

#define SCLK_L  PORTD &= ~(1<<PD3)  /* SCLK VNC1L */
#define SCLK_H  PORTD |=  (1<<PD3)

#define SDI_L   PORTD &= ~(1<<PD4)  /* SDI  VNC1L */
#define SDI_H   PORTD |=  (1<<PD4)

#define SDO_VNC (PIND & (1<<PD5)) 	/* SDO  VNC1L */

#define CS_VNC_L PORTD &= ~(1<<PD6) /* CS   VNC1L */
#define CS_VNC_H PORTD |=  (1<<PD6)

#define RE_VNC_L PORTD &= ~(1<<PD7) /* RS   VNC1L */
#define RE_VNC_H PORTD |=  (1<<PD7)


// Variablenbereich
unsigned char read_e, read_cr;

// ***********************************************************************
// VNC1L - USB HOST CONTROLLER - SPI-MODE (MC: Master, VNC1L: SLAVE)
// VDIP1: Jumper-Set: J3 - Pulldown / J4 - Pullup
//        VDAP Firmware muss installiert sein, mit Version 3.68 getestet
// ***********************************************************************

// ***********************************************************************
// VNC1L - Intialisierung SPI-Interface und VNC1L
// SPI Mode
// ***********************************************************************
void vnc_init(void) {
  CS_VNC_L;       // SPI-Interface inaktiv
  SCLK_L;         // Ruhezustand der SPI-Leitungen herstellen
  SDI_H;
  RE_VNC_L;       // Hardware-Reset VNC1L
  _delay_ms(100); // Pause
  RE_VNC_H;       // VNC1L starten
}

// ***********************************************************************
// VNC1L - Senden eines Zeichens
// SPI Mode
// bergabe: wr_byte   : Auszugebendes Zeichen
// Rckgabe: statusbit : 0=Accept (ok) 1=Reject (nicht ok)
// ***********************************************************************
unsigned char vnc_wr_byte(unsigned char wr_byte) {
unsigned char stelle, status;
  CS_VNC_H;        // SPI-Interface aktiv
  SDI_H;           // Startbit (immer 1)
  SCLK_H;  SCLK_L; // L/H Flanke
  SDI_L;           // RW-Bit (0=Write)
  SCLK_H;  SCLK_L; // L/H Flanke
  SDI_L;           // ADDR (0=Daten)
  SCLK_H;  SCLK_L; // L/H Flanke
  // 8 Datenbits senden (MSB zuerst)
  for ( stelle = 0x80; stelle; stelle >>= 1 ) {
    if (wr_byte & stelle) SDI_H; else SDI_L;
    SCLK_H;  SCLK_L; // L/H Flanke
  }
  // Statusbit einlesen
  status = SDO_VNC; // Statusbit (0=Accept, 1=Reject)
  // keine Prfung
  SCLK_H;  SCLK_L; // L/H Flanke
  // End Sequenz
  CS_VNC_L;         // SPI-Interface inaktiv
  SCLK_H;  SCLK_L;  // L/H Flanke
  _delay_ms(100);   // Pause nur zum Test
  return status;
} 

// ***********************************************************************
// VNC1L - Empfangen eines Zeichens
// SPI Mode
// Rckgabe: Datenbyte
// ***********************************************************************
unsigned char vnc_rd_byte(void) {
unsigned char stelle, daten, status;
  daten  = 0;
  CS_VNC_H;        // SPI-Interface aktiv
  SDI_H;           // Startbit (immer 1)
  SCLK_H;  SCLK_L; // L/H Flanke
  SDI_H;           // RW-Bit (1=Read)
  SCLK_H;  SCLK_L; // L/H Flanke
  SDI_L;           // ADDR (0=Daten, 1=SPI-Statusinformation) 
  SCLK_H;  SCLK_L; // L/H Flanke
  // 8 Datenbits empfangen (MSB zuerst)
  for ( stelle = 0x80; stelle; stelle >>= 1 ) {
    if (SDO_VNC) daten |= stelle;
    SCLK_H;  SCLK_L; // L/H Flanke
  }
  // Statusbit einlesen
  status = SDO_VNC; // Statusbit (0=New Data, 1=Old Data)
  // keine Prfung
  SCLK_H;  SCLK_L; // L/H Flanke
  // End Sequenz
  CS_VNC_L;        // SPI-Interface inaktiv
  SCLK_H;  SCLK_L; // L/H Flanke
  _delay_ms(100);  // Pause nur zum Test
  return daten;
} 

// ********************************************************************
// Hauptprogramm
// ********************************************************************
int main (void) {
  // Portdefinitionen ANPASSEN
  // LED
  DDRB  |= (1 << PB0); // PB0 als Ausgang
  LED_D1_AUS;          // LED aus
  // VNC1L
  DDRD  |= ((1 << PD3) | (1 << PD4) | (1 << PD6) | (1 << PD7)); // PD3,PD4,PD6,PD7 als Ausgang
  DDRD  &= ~(1 << PD5); // PD5 als Eingang

  vnc_init();    // VNC1L-Initialisierung
  // VNC1L Bereitschaft abwarten indem auf > und CR geprft wird 
  while(vnc_rd_byte() != '>');
  vnc_rd_byte(); // CR nach dem > jetzt noch abholen
  // VNC1L hat Promptzeichen > gesendet und kann jetzt neue Kommandos annehmen
  while(1) {
    LED_D1_AUS;              // LED aus
    vnc_wr_byte('E');        // Echo senden
    vnc_wr_byte(0x0D);       // CR senden
    _delay_ms(500);          // Pause
    read_e  = vnc_rd_byte(); // Echo empfangen
    read_cr = vnc_rd_byte(); // CR empfangen
    if ((read_e == 'E') && (read_cr == 0x0D)) LED_D1_EIN; // LED ein wenn Echo empfangen wurde
    _delay_ms(1000); // Pause
  }
}
